import cloneDeep from "clone-deep";
import actions from "../../../src/store/actions";
import state from "../../mocks/state";
import Store from "../../../src/store/index";

const store = Store(state);

describe("actions", () => {
  const data = {
    id: "element1",
    element: {
      html: "foo"
    }
  };
  let stub;

  beforeEach(() => {
    stub = jest.fn();
    store.commit = stub;
  });

  afterEach(() => {
    store.replaceState(cloneDeep(state));
  });

  describe("editContent()", () => {
    describe("when the passed element doesn't exit in the state yet", () => {
      it("commits addElement with the passed data to the store", () => {
        actions.editContent(store, data);
        expect(stub).toHaveBeenCalledWith("addElement", data);
      });
    });

    describe("when the passed element exists in the state", () => {
      it("commits updateElement with the passed to the store", () => {
        const props = {
          id: "element7",
          element: {
            html: "foo"
          }
        };
        actions.editContent(store, props);
        expect(stub).toHaveBeenCalledWith("updateElement", props);
      });
    });

    it("commits editContent on the store", () => {
      actions.editContent(store, data);
      expect(stub).toHaveBeenCalledWith("editContent", data);
    });

    it("commits setCurrentlyEditableContent on the store with null", () => {
      actions.editContent(store, data);
      expect(stub).toHaveBeenCalledWith("setCurrentlyEditableContent", null);
    });

    it("commits setContentOverlayVisibility on the store with false", () => {
      actions.editContent(store, data);
      expect(stub).toHaveBeenCalledWith("setContentOverlayVisibility", false);
    });
  });

  describe("chooseElement()", () => {
    it("commits editContent on the store", () => {
      actions.chooseElement(store, data);
      expect(stub).toHaveBeenCalledWith("editContent", data);
    });

    it("commits setCurrentlyEditableContent on the store with null", () => {
      actions.chooseElement(store, data);
      expect(stub).toHaveBeenCalledWith("setCurrentlyEditableContent", null);
    });

    it("commits setContentOverlayVisibility on the store with false", () => {
      actions.chooseElement(store, data);
      expect(stub).toHaveBeenCalledWith("setContentOverlayVisibility", false);
    });
  });

  describe("showContentOverlay()", () => {
    it("commits setCurrentlyEditableContent on the store with data", () => {
      actions.showContentOverlay(store, data);
      expect(stub).toHaveBeenCalledWith("setCurrentlyEditableContent", data);
    });

    it("commits setContentOverlayVisibility on the store with true", () => {
      actions.showContentOverlay(store, data);
      expect(stub).toHaveBeenCalledWith("setContentOverlayVisibility", true);
    });
  });

  describe("closeContentOverlay()", () => {
    it("commits setCurrentlyEditableContent on the store with null", () => {
      actions.closeContentOverlay(store);
      expect(stub).toHaveBeenCalledWith("setCurrentlyEditableContent", null);
    });

    it("commits setContentOverlayVisibility on the store with false", () => {
      actions.closeContentOverlay(store);
      expect(stub).toHaveBeenCalledWith("setContentOverlayVisibility", false);
    });
  });

  describe("save()", () => {
    it("makes a request", () => {
      window.fetch = jest.fn(() => new Promise(resolve => resolve()));
      actions.save({ commit: stub, state });
      expect(window.fetch).toHaveBeenCalled();
    });

    describe("successful request", () => {
      const message = "foo";
      const config = { foo: "bar" };

      it("commits 'success' with the given message", done => {
        window.fetch = jest.fn(
          () =>
            new Promise(resolve =>
              resolve({
                ok: true,
                json: () =>
                  new Promise(res =>
                    res({
                      ok: true,
                      success: true,
                      message,
                      config
                    })
                  )
              })
            )
        );
        actions.save({ commit: stub, state });
        setTimeout(() => {
          expect(stub).toHaveBeenCalledWith("success", message);
          done();
        });
      });

      it("commits 'resetConfig' with the given config", done => {
        window.fetch = jest.fn(
          () =>
            new Promise(resolve =>
              resolve({
                ok: true,
                json: () =>
                  new Promise(res =>
                    res({
                      ok: true,
                      success: true,
                      message,
                      config
                    })
                  )
              })
            )
        );
        actions.save({ commit: stub, state });
        setTimeout(() => {
          expect(stub).toHaveBeenCalledWith("resetConfig", config);
          done();
        });
      });

      describe("without passed config", () => {
        window.fetch = jest.fn(
          () =>
            new Promise(resolve =>
              resolve({
                ok: true,
                json: () =>
                  new Promise(res =>
                    res({
                      ok: true,
                      success: true,
                      message
                    })
                  )
              })
            )
        );

        it("doesnt call commit with 'resetConfig'", done => {
          actions.save({ commit: stub, state });
          setTimeout(() => {
            expect(stub).not.toHaveBeenCalledWith("resetConfig");
            done();
          });
        });
      });
    });

    describe("unsuccessful request", () => {
      const message = "foo";

      it("commits 'error' with the given message", done => {
        window.fetch = jest.fn(
          () =>
            new Promise(resolve =>
              resolve({
                ok: true,
                json: () =>
                  new Promise(res =>
                    res({
                      ok: true,
                      success: false,
                      message
                    })
                  )
              })
            )
        );
        actions.save({ commit: stub, state });
        setTimeout(() => {
          expect(stub).toHaveBeenCalledWith("error", message);
          done();
        });
      });
    });

    describe("a not okayish response", () => {
      it("commits 'error' with a static message", done => {
        window.fetch = jest.fn(
          () =>
            new Promise(resolve =>
              resolve({
                ok: false
              })
            )
        );
        actions.save({ commit: stub, state });
        setTimeout(() => {
          expect(stub).toHaveBeenCalledWith(
            "error",
            "Saving failed. Please try again."
          );
          done();
        });
      });
    });

    describe("not parseable response", () => {
      it("commits 'error' with a static message", done => {
        window.fetch = jest.fn(
          () =>
            new Promise(resolve =>
              resolve({
                ok: true,
                json: () => new Promise((res, reject) => reject())
              })
            )
        );
        actions.save({ commit: stub, state });
        setTimeout(() => {
          expect(stub).toHaveBeenCalledWith(
            "error",
            "Saving failed. Please try again."
          );
          done();
        });
      });
    });
  });
});
