import cloneDeep from "clone-deep";
import mutations from "../../../src/store/mutations";
import state from "../../mocks/state";
import Store from "../../../src/store/index";

const store = Store(state);

describe("mutations", () => {
  afterEach(() => {
    store.replaceState(cloneDeep(state));
  });

  it("move() moves an item in array from a given index to another index", () => {
    const arr = [1, 2, 3];
    mutations.move(store.state, { arr, to: 1, index: 0 });
    expect(arr).toEqual([2, 1, 3]);
  });

  it("updateSectionOrder() updates the sections array", () => {
    mutations.updateSectionOrder(store.state, [1, 2, 3]);
    expect(store.state.sections).toEqual([1, 2, 3]);
  });

  it("changeLayout() updates the layout of a section", () => {
    const indexSection = 3;
    const selectedLayout = "foo";

    mutations.changeLayout(store.state, {
      indexSection,
      selectedLayout
    });

    expect(store.state.sections[indexSection].layout).toEqual(selectedLayout);
  });

  describe("deleteSection()", () => {
    describe("when it is the last section", () => {
      const mockedState = {
        sections: [
          {
            layout: "foo",
            content: "bar"
          }
        ]
      };

      it("resets the section", () => {
        mutations.deleteSection(mockedState, 0);
        expect(mockedState.sections[0].layout).toEqual(null);
        expect(mockedState.sections[0].content).toEqual({});
      });
    });

    describe("when it is not the last section", () => {
      const indexSection = 0;
      const { length } = state.sections;
      const section = state.sections[indexSection];

      it("removes the section from the array", () => {
        mutations.deleteSection(store.state, { indexSection });

        expect(store.state.sections.length).toBe(length - 1);
        expect(store.state.sections[indexSection]).not.toEqual(section);
      });
    });
  });

  it("addSection() adds a section at a given index", () => {
    const index = 1;
    const sectionsLength = state.sections.length;

    mutations.addSection(store.state, index);

    expect(store.state.sections[index].layout).toEqual(null);
    expect(store.state.sections[index].content).toEqual({});

    expect(store.state.sections.length).toBe(sectionsLength + 1);
  });

  it("updateContentOrder() updates the content array", () => {
    const indexSection = 1;
    const slotId = "1";
    const value = [null, "element7", "element8"];

    mutations.updateContentOrder(store.state, {
      indexSection,
      slotId,
      value
    });

    expect(store.state.sections[indexSection].content[slotId]).toEqual(value);
  });

  describe("editContent()", () => {
    describe("when there was no content set yet", () => {
      const indexSection = 0;
      const slotId = "1";
      const indexSectionBlock = 0;
      const newId = "element9";

      it("adds an array with the content at 0", () => {
        store.state.currentlyEditableContentElement = {
          indexSection,
          slotId,
          indexSectionBlock
        };

        mutations.editContent(store.state, {
          id: newId,
          element: {
            html: "foo",
            editUrl: "bar"
          }
        });

        expect(store.state.sections[indexSection].content[slotId]).toEqual([
          "element9"
        ]);
      });
    });

    describe("when there was already content set", () => {
      const indexSection = 1;
      const slotId = "1";
      const indexSectionBlock = 0;
      const id = "element9";

      it("updates the content at a given index", () => {
        store.state.currentlyEditableContentElement = {
          indexSection,
          slotId,
          indexSectionBlock
        };

        mutations.editContent(store.state, {
          id,
          element: {
            html: "foo",
            editUrl: "bar"
          }
        });

        expect(store.state.sections[indexSection].content[slotId]).toEqual([
          "element9",
          null,
          "element8"
        ]);
      });
    });
  });

  describe("deleteContent()", () => {
    describe("the last one", () => {
      const indexSection = 3;
      const slotId = "1";
      const indexSectionBlock = 0;

      it("removes the key from the content object", () => {
        mutations.deleteContent(store.state, {
          indexSection,
          slotId,
          indexSectionBlock
        });

        expect(store.state.sections[indexSection].content[slotId]).toEqual(
          undefined
        );
      });
    });

    describe("not the last one", () => {
      const indexSection = 1;
      const slotId = "1";
      const indexSectionBlock = 0;
      const arr = state.sections[indexSection].content[slotId];
      const { length } = arr;
      const id = arr[indexSectionBlock];

      it("removes the content id from the array", () => {
        mutations.deleteContent(store.state, {
          indexSection,
          slotId,
          indexSectionBlock
        });

        expect(store.state.sections[indexSection].content[slotId].length).toBe(
          length - 1
        );
        expect(
          store.state.sections[indexSection].content[slotId][indexSectionBlock]
        ).not.toEqual(id);
      });
    });
  });

  it("addContent() adds a content block at a given index", () => {
    const indexSection = 1;
    const slotId = "1";
    const indexSectionBlock = 0;
    const { length } = state.sections[indexSection].content[slotId];

    mutations.addContent(store.state, {
      indexSection,
      slotId,
      indexSectionBlock
    });

    expect(
      store.state.sections[indexSection].content[slotId][indexSectionBlock]
    ).toEqual(null);

    expect(store.state.sections.length).toBe(length + 1);
  });

  it("reset() sets the state to the given state", () => {
    const newState = {
      sections: ["section"],
      elements: ["element"],
      layouts: ["layout"],
      saveUrl: "saveUrl"
    };
    mutations.reset(store.state, newState);
    expect(store.state.sections).toEqual(cloneDeep(newState).sections);
    expect(store.state.elements).toEqual(cloneDeep(newState).elements);
    expect(store.state.layouts).toEqual(cloneDeep(newState).layouts);
    expect(store.state.saveUrl).toEqual(cloneDeep(newState).saveUrl);
  });

  it("setContentOverlayVisibility() sets the correct value to state.contentOverlayVisible", () => {
    const visible = true;
    mutations.setContentOverlayVisibility(store.state, visible);
    expect(store.state.contentOverlayVisible).toBe(visible);
  });

  it("setCurrentlyEditableContent() sets the correct value to state.currentlyEditableContentElement", () => {
    const element = "foo";
    mutations.setCurrentlyEditableContent(store.state, element);
    expect(store.state.currentlyEditableContentElement).toBe(element);
  });

  it("setActiveGroupInContentOverlay() sets the correct value to state.activeGroupInContentOverlay", () => {
    const groupName = "foo";
    mutations.setActiveGroupInContentOverlay(store.state, groupName);
    expect(store.state.activeGroupInContentOverlay).toBe(groupName);
  });

  it("addElement() adds a new element to the elements object", () => {
    const element = "element1";
    expect(state.elements[element]).toBeFalsy();
    mutations.addElement(store.state, { id: element });
    expect(state.elements[element]).toBeFalsy();
  });

  it("updateElement() overwrites the html of an edited element", () => {
    const element = "element7";
    const newHTML = Date.now().toString();
    mutations.updateElement(Store(state).state, {
      id: element,
      element: {
        html: newHTML
      }
    });
    expect(state.elements[element].html).toBe(newHTML);
  });

  it("error() sets the message object with the given msg", () => {
    const msg = "foo";
    mutations.error(store.state, msg);
    expect(store.state.message).toEqual({ type: "error", message: msg });
  });

  it("success() sets the message object with the given msg", () => {
    const msg = "foo";
    mutations.success(store.state, msg);
    expect(store.state.message).toEqual({ type: "success", message: msg });
  });

  it("removeMessage() sets the message object to false", () => {
    mutations.error(store.state, "foo");
    mutations.removeMessage(store.state);
    expect(store.state.message).toEqual(false);
  });
});
